/**************************************************************************************************************
 *
 * I2C-Funktionsbibliothek
 *
 *************************************************************************************************************/

#include <p18cxxx.h>
#include <p18F452.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "defines.h"
#include "serial.h"
#include "i2c.h"
#include  "defines.h"

char OutWert = 0;																									// Ausgabewert fr den Output-Port
char ReadErr = FALSE;

/***********************************************************************************
 *
 * Schaltet SDA und SCL als Eingang und macht die Schnittstelle startklar
 *
 ***********************************************************************************/
void i2c_init (void)
 {
  TRISC |= SDA_PORT;                                             // SDA = RC4 = Input
  TRISC |= SCL_PORT;                                             // SCL = RC3 = Input

  SSPCON1 = 0b00001000;                                           // Master-Mode, Clock : SSPADD
  SSPCON1bits.SSPEN = 1;                                          // Enable
  PIR1bits.SSPIF = 0;                                             // Interruptflag rcksetzen
  //SSPADD = 0x39;                                                  // I2C-Clock = 100Khz
  SSPADD = 0x20;                                                  // I2C-Clock = 100Khz

  TRISB &=0b11101111;											  // B4 als Output
  PORTB |= 0b00010000;											  // B4 auf High (Versorgt LM75)
 }

/*************************************************************************************
 *
 * Adressiert einen Slave, liefert TRUE falls dieser mit ACK antwortet, sonst FALSE
 * Richtung TRUE = Schreiben zum Slave, FALSE = Lesen vom Slave
 *
 *************************************************************************************/
char i2c_open (char Adresse, char Richtung)
 {
  #define I2CTimeout  100

  char Timeout = 0;                                         // Timeout-Zhler

  SSPCON2bits.SEN = 1;                                                  // Start-Condition generieren

  // Warten, bis die Start-Condition fertig ist
  while ( (!PIR1bits.SSPIF) && (Timeout < I2CTimeout) )
   {
    DelayUs (80);
    Timeout++;
   }

  PIR1bits.SSPIF = 0;                                                // Flag rcksetzen

  if (Timeout >= I2CTimeout) return (FALSE);

  if (Richtung) SSPBUF = Adresse; else SSPBUF = Adresse+1;  // Adresse senden

  // Warten auf das setzen des Interrupt-Flags
  Timeout = 0;
  while ( (!PIR1bits.SSPIF) && (Timeout < I2CTimeout) )
   {
    DelayUs (80);
    Timeout++;
   }

  PIR1bits.SSPIF = 0;                                                // Flag rcksetzen

  if (Timeout >= I2CTimeout) return (FALSE);

  if (!(SSPCON2bits.ACKSTAT)) return (TRUE); else                       // Prfen auf ACK vom Slave
                              return (FALSE);
 }

/*************************************************************************************
 *
 * Erzeugt die Stop-Condition
 *
 *************************************************************************************/
void i2c_close (void)
 {
  #define I2CTimeout  100

  char Timeout = 0;

  SSPCON2bits.PEN = 1;                                                  // Start-Condition generieren

  // Warten, bis die Start-Condition fertig ist
  while ( (SSPCON2bits.PEN) && (Timeout < I2CTimeout) )
   {
    DelayUs (50);
    Timeout++;
   }

  PIR1bits.SSPIF = 0;                                                // Flag rcksetzen
 }


/*************************************************************************************
 *
 * Schreibt ein Byte via i2C, liefert TRUE bei ACK vom Slave, sonst FALSE
 *
 *************************************************************************************/
char i2c_writebyte (unsigned char wert)
 {
  #define I2CTimeoutWrite  500

  int Timeout = 0;

  SSPBUF = wert;

  // Warten auf das setzen des Interrupt-Flags
  while ( (!PIR1bits.SSPIF) && (Timeout < I2CTimeoutWrite) )
   {
    DelayUs (50);
    Timeout++;
   }

  PIR1bits.SSPIF = 0;                                                // Flag rcksetzen

  if (!(SSPCON2bits.ACKSTAT)) return (TRUE); else                       // Prfen auf ACK vom Slave
                  return (FALSE);

 }

/***********************************************************************************
 *
 * Liest ein Byte via i2C, erzeugt bei ACK ein Acknowledge
 *
 ***********************************************************************************/
char i2c_readbyte (char ACKn)
 {
  #define I2CTimeoutRead  500

  int Timeout = 0;

   SSPCON2bits.RCEN = 1;                                  // Receive einschalten

   while ( (SSPCON2bits.RCEN) && (Timeout < I2CTimeoutRead) )
    {
     DelayUs (30);
     Timeout++;
    }

   SSPCON2bits.ACKEN = ACKn;                              // Acknowledgeerzeugung bei Bedarf einschalten

   // Warten, bis ein Byte drin ist
  Timeout = 0;
  while ( (!PIR1bits.SSPIF) && (Timeout < I2CTimeoutRead) )
   {
    DelayUs (40);
    Timeout++;
   }

   PIR1bits.SSPIF = 0;                                 // Flag rcksetzen

   return (SSPBUF);                           // Wert bergeben
 }


/**************************************************************************
 *
 * Liefert True, wenn der Slave vorhanden ist
 *
 **************************************************************************/
char i2c_BusScan (char Adresse)
 {
  char Flag;

  Flag = i2c_open (Adresse, WRITE);         // Slave ansprechen
  DelayUs (800);                            // Kurze Pause
  i2c_close ();                             // Verbindung wieder beenden
  return (Flag);                            // Ergebniss zurck
 }

/********************************************************************
 *
 * Setzt den Output-Port PCF8674
 *
 ********************************************************************/
char I2C_Output (char Adresse, char Wert)
 {

  // Slave adressieren zum Schreiben
  if (!(i2c_open (Adresse, WRITE)))
   {
     // Verbindung beenden
     i2c_close ();
     return (FALSE);
   }

  // Datum schreiben
  if (!(i2c_writebyte (Wert)))
   {
    // Verbindung beenden
    i2c_close ();
    return (FALSE);
   }

  // Verbindung beenden
  i2c_close ();

  return (TRUE);
 }

/********************************************************************
 *
 * Liest den Input-Port
 *
 ********************************************************************/
char I2C_Input (char Adresse)
 {
  char ret;

  // Slave adressieren zum Schreiben
  if (!(i2c_open (Adresse, WRITE)))
   {
     // Verbindung beenden
     i2c_close ();
     return (FALSE);
   }

  // Wert schreiben -> Alle Portpins auf Eingang schalten
  if (!(i2c_writebyte (0xFF)))
   {
    // Verbindung beenden
    i2c_close ();
    return (FALSE);
   }

  // Verbindung beenden
  i2c_close ();

  // Mistkacke Ende

  // Slave adressieren zum Lesen
  if (!(i2c_open (Adresse, READ)))
   {
     // Verbindung beenden
     i2c_close ();
     return (FALSE);
   }

  // Ein Datum lesen
  ret = i2c_readbyte (FALSE);

  // Verbindung beenden
  i2c_close ();

  return (ret);
 }

/************************************
 *
 * Liest einen Temperatursensor aus
 *
 ************************************/
int i2c_ReadTemp (char Adresse)
 {
  int High;
  int Low;
  int Wert;
  int ret;

/*
  PORTB &= ~0b00010000;											 // B4 auf Low (Versorgt LM75)
  DelayUs (5000);	
  PORTB |= 0b00010000;											 // B4 auf High (Versorgt LM75)
  DelayUs (5000);	
*/
    // Slave adressieren zum Lesen
    if (!(i2c_open (Adresse, READ)))
     {
       // Verbindung beenden
       i2c_close ();
       return (-70);			// Fehler anzeigen
     }

    // Datum auslesen
    High = i2c_readbyte (TRUE);
    Low = i2c_readbyte (FALSE);

    // Verbindung beenden
    i2c_close ();

    // Wert zusammenbauen
    Wert = (High*256 + Low) >> 7;       // Integer zusammensetzen und rechtsbndig machen
    ret = Wert;
    ret &= 0b0000000111111111;		    // Untere 9 Bits ausmarkieren		 

    // MSB passend setzen, also bei negativen Werten 2er-Komplement bilden
    if (Wert & 0b0000000100000000)
     {
      ret |= ~0b0000000111111111;		    // Untere 9 Bits setzen
     }

  // Jedes Bit entspricht 0.5C, also ganze C zurckliefern
   ret  /= 2;

  return (ret);

 }
